﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace Utility.Timer
{
    //通过win api QueryPerformanceCounter实现的timer
    public class Win32Timer : ITimer
    {
        [DllImport("Kernel32.dll")]
        extern static short QueryPerformanceCounter(ref long count);
        [DllImport("Kernel32.dll")]
        extern static short QueryPerformanceFrequency(ref long frequency);

        private bool isPause;
        private long currentTick;
        private long lastCount;
        private long frequency;
        private long curCount;


        public int Step
        {
            get;
            private set;
        }

        public long FrameTime
        {
            get;
            private set;
        }

        public Win32Timer()
        {
            Step = 0;
            FrameTime = 0;

            lastCount = 0;
            currentTick = 0;

            isPause = true;

            QueryPerformanceFrequency(ref frequency);
        }

        public void Start()
        {
            Stop();

            isPause = false;

            QueryPerformanceFrequency(ref frequency);
            QueryPerformanceCounter(ref lastCount);
        }

        public void Stop()
        {
            Step = 0;
            FrameTime = 0;

            lastCount = 0;
            currentTick = 0;

            isPause = true;
        }

        public void Pause()
        {
            isPause = true;
        }

        public void Resume()
        {
            isPause = false;
        }

        public void Update()
        {
            long Now = CalcTick();
            if (FrameTime != 0)
            {
                Step = (int)(Now - FrameTime);
            }
            FrameTime = Now;
        }

        public long CalcTick()
        {
            long Count = 0;
            QueryPerformanceCounter(ref Count);
            curCount = Count;

            long StepTick = Count - lastCount;
            lastCount = Count;

            if (!isPause)
            {
                currentTick += StepTick;
            }

            return (currentTick * 1000 / frequency);
        }

        public long GetTick()
        {
            return (currentTick * 1000 / frequency);
        }
    }
}
